var Adhoc = {
    status: {
        sessionid: null,
        cmdNode: null,
        queryJID: null
    },

    addNote: function (elem, text, type) {
        if (!type) {
           type = "info";
        }
        text = text.replace(/\n/g, "<br/>");
        $(elem).append("<p class='" + type + "Note'>" + text + "</p>");
    },

    addForm: function (elem, x) {
        var form = $("<form action='#'/>");
        form.submit(function(event) {
            Adhoc.executeCommand("execute", Adhoc.serializeToDataform('form'),
                function(e) { Adhoc.displayResult(elem, e) });
            event.preventDefault();
        });
        var fieldset = $("<fieldset/>");
        form.append(fieldset);
        if ($(x).find("title").length > 0)
            $("<legend/>").text($(x).find("title").text()).appendTo(fieldset);
        if ($(x).find("instructions").length > 0)
            $("<p/>").text($(x).find("instructions").text()).appendTo(fieldset);
        $(x).find("field").each(function() {
            var item = null;
            var type = $(this).attr("type");
            if($(this).attr("label")) {
                $("<label/>").text($(this).attr("label")).attr("for", $(this).attr("var")).appendTo(fieldset);
                $("<br/>").appendTo(fieldset);
            }
            switch(type) {
            case "hidden":
                item = $("<input type='hidden'/>");
                break;
            case "boolean":
                item = $("<input type='checkbox'/>");
                break;
            case "text-multi":
                item = $("<textarea rows='10' cols='70'/>");
                break;
            case "text-single":
                item = $("<input type='text'/>");
                break;
            case "fixed":
                item = $("<input type='text'/>").attr("readonly",true);
                break;
            case "jid-multi":
                item = $("<textarea rows='10' cols='70'/>");
                break;
            case "jid-single":
                item = $("<input type='text'/>");
                break;
            case "list-multi":
                item = $("<select multiple='multiple'/>");
                $(this).find("option").each(function() {
                    $("<option/>").val($(this).find("value").text()).text($(this).attr("label")).appendTo(item);
                });
                break;
            case "list-single":
                item = $("<select/>");
                $(this).find("option").each(function() {
                    $("<option/>").val($(this).find("value").text()).text($(this).attr("label")).appendTo(item);
                });
                break;
            case "text-private":
                item = $("<input type='password'/>");
                break;
            default:
                item = $("<input/>");
            }
            item.addClass("df-item");
            if ($(this).find("value").length > 0) {
                var value = null;
                if ((type == "text-multi") || (type == "jid-multi")) {
                    value = "";
                    $(this).find("value").each(function() {
                        value = value + $(this).text() + "\n";
                    });
                    item.val(value);
                } else if (type == "list-multi") {
                    $(this).children("value").each(function() {
                        item.children('option[value="' + $(this).text() + '"]').each(function() {
                            $(this).attr("selected", "selected");
                });
                    });
                } else {
                    item.val($(this).find("value").text());
                }
            }
            if ($(x).attr("type") == "result")
                item.attr("readonly", true);
            if ($(this).attr("var")) {
                item.attr("name", $(this).attr("var"));
                item.attr("id", $(this).attr("var"));
            }
            fieldset.append(item);
            if (type != "hidden")
                fieldset.append("<br/>");
        });
        $(elem).append(form);
    },

    serializeToDataform: function (form) {
        st = $build("x", {"xmlns": "jabber:x:data", "type": "submit"});
        $(form).find(".df-item").each(function(){
            st.c("field", {"var": $(this).attr("name")});
            if (this.nodeName.toLowerCase() == "select" && this.multiple) {
                for (var i = 0; i < this.options.length; i++)
                    if (this.options[i].selected)
                        st.c("value").t(this.options[i].text).up();
            } else if (this.nodeName.toLowerCase() == "textarea") {
                var sp_value = this.value.split(/\r?\n|\r/g);
                for(var i = 0; i < sp_value.length; i++)
                    st.c("value").t(sp_value[i]).up();
            } else if (this.nodeName.toLowerCase() == "input" && this.type == "checkbox") {
                if (this.checked) {
                    st.c("value").t("1");
                } else {
                    st.c("value").t("0");
                }
            } else {
                // if this has value then
                st.c("value").t($(this).val()).up();
            }
            st.up();
        });
        st.up();
        return st.tree();
    },

    displayResult: function (elem, result) {
        var status = $(result).find("command").attr("status");
        var kinds = {'prev': 'Prev', 'next': 'Next', 'complete': 'Complete'};

        $(elem).empty();
        $(result).find("command > *").each(function(index, e) {
            if ($(e).is("note")) {
                Adhoc.addNote(elem, $(e).text(), $(e).attr("type"));
            } else if ($(e).is("x[xmlns=jabber:x:data]")) {
                Adhoc.addForm(elem, e);
            }
        });
        if (status == "executing") {
            for (kind in kinds) {
                input = $("<input type='button' disabled='disabled' value='" + kinds[kind] + "'/>").click(function() {
                    if (kind == 'prev')
                        Adhoc.executeCommand(kind, false, function(e) { Adhoc.displayResult(elem, e) });
                    else
                        Adhoc.executeCommand(kind, Adhoc.serializeToDataform('form'), function(e) { Adhoc.displayResult(elem, e) });
                });
                if ($(result).find('actions ' + kind).length > 0)
                    input.removeAttr("disabled");
                $(elem).append(input);
            }

            $("<input type='button' id='executeButton' value='Execute'/>").click(function() {
                Adhoc.executeCommand("execute", Adhoc.serializeToDataform('form'), function(e) { Adhoc.displayResult(elem, e) });
            }).appendTo(elem);

            $("<input type='button' value='Cancel'/>").click(function() {
                Adhoc.cancelCommand(function(e) { Adhoc.displayResult(elem, e) });
            }).appendTo(elem);
        } else {
            input = $("<input type='button' value='Start over'/>").bind("click", function() {
                $(elem).empty();
                Adhoc.status.sessionid = null;
                Adhoc.status.cmdNode = null;
                Adhoc.getCommandNodes(elem);
            });
            $(elem).append(input);
        }
    },

    runCommand: function (item, callback) {
        Adhoc.status.cmdNode = $(item).attr("id"); // Save node of executed command (in global var)
        Adhoc.executeCommand("execute", false, function(result) {
            Adhoc.status.sessionid = $(result).find("command").attr("sessionid");
            callback(result);
        });
    },

    executeCommand: function (type, childs, callback) {
        if (Adhoc.status.sessionid)
            var execIQ = $iq({ type: "set", to: Adhoc.status.queryJID, id: connection.getUniqueId() })
                .c("command", { xmlns: Strophe.NS.ADHOC, node: Adhoc.status.cmdNode, sessionid: Adhoc.status.sessionid, action: type });
        else
            var execIQ = $iq({ type: "set", to: Adhoc.status.queryJID, id: connection.getUniqueId() })
                .c("command", { xmlns: Strophe.NS.ADHOC, node: Adhoc.status.cmdNode, action: type });
        if (childs)
            execIQ.cnode(childs);
            connection.sendIQ(execIQ, callback);
    },

    cancelCommand: function (callback) {
        Adhoc.executeCommand("cancel", false, callback);
        Adhoc.status.cmdNode = null
        Adhoc.status.sessionid = null;
    },

    getCommandNodes: function (elem) {
        var nodesIQ = $iq({ type: "get", to: Adhoc.status.queryJID, id: connection.getUniqueId() }).c("query", {xmlns: Strophe.NS.DISCO_ITEMS, node: Strophe.NS.ADHOC});
        connection.sendIQ(nodesIQ, function(result) {
            var items = $("<ul></ul>");
            $(elem).append(items);
            $(result).find("item").each(function(index, e) {
                $("<li></li>").append($("<a href='#' id='" + $(e).attr("node") + "'>" + $(e).attr("name") + "</a>").click(function (event) {
                    Adhoc.runCommand(this, function (result) { Adhoc.displayResult(elem, result); });
                    event.preventDefault();
                })).appendTo(items);
            });
        });
    },

    checkFeatures: function (elem, jid) {
        if (Adhoc.status.sessionid)
            Adhoc.cancelCommand();
        Adhoc.status.queryJID = jid;
        var featureIQ = $iq({ type: "get", to: Adhoc.status.queryJID, id: connection.getUniqueId() }).c("query", {xmlns: Strophe.NS.DISCO_INFO});
        $(elem).empty();
        connection.sendIQ(featureIQ,
            function(result) { /* Callback */
                if ($(result).find("feature[var='" + Strophe.NS.ADHOC + "']").length > 0) {
                    Adhoc.getCommandNodes(elem);
                } else {
                    $(elem).append("<p>" + Adhoc.status.queryJID + " does NOT support AdHoc commands</p>");
                }
            },
            function(result) { /* Errback */
                $(elem).append("<p>Couldn't get list of supported features</p>");
            }
        );
    }

}
